$wnd.rstudio.runAsyncCallback15("function IGq(){}\nfunction eGq(a){this.a=a}\nfunction GGq(a){this.a=a}\nfunction LGq(a){this.a=a}\nfunction NGq(a){this.a=a}\nfunction RHq(a){this.a=a}\nfunction nYi(a){this.a=a}\nfunction cGq(a){oCi.call(this,a)}\nfunction NFq(a){XFq(a,new LGq(a))}\nfunction g2e(c,a,b){return c.slice(a,b)}\nfunction f2e(a,b){return g2e(a,b,a.length||0)}\nfunction WWi(a,b){gWi();(nQe(b),b)&&WFq(a.a,pVB,'')}\nfunction UFq(a,b){o5h(a.d,new Ffr(b,true,true),false)}\nfunction RFq(a){h2t(a.o,new GGq(a),new IGq,HFq.A8b())}\nfunction SFq(a){LWi(a.b,HFq.H8b(),HFq.H8b(),new NGq(a))}\nfunction TFq(a){a.K.S7b();gn((_m(),$m),new eGq(a))}\nfunction XFq(a,b){a.c!=null?KGq(b,a.c):iBm(a.k,new gGq(a,b))}\nfunction gGq(a,b){Puf();this.a=a;this.b=b;Ruf.call(this)}\nfunction KGq(a,b){UFq(a.a,'devtools::load_all(\"'+b+'\")')}\nfunction iBm(a,b){GFm(a,yzz,'devtools_load_all_path',new xL,b)}\nfunction LWi(a,b,c,d){wWi(a,b,c,OYi(N_o(a.j.b),'roxygen'),jWi(a,'roxygen'),false,new nYi(d))}\nfunction KFq(a,b){var c,d;XOi(a.s.g,null);d=b.outputs;(d.length||0)>Xwy&&(d=f2e(d,(d.length||0)-Xwy));for(c=0;c<(d.length||0);c++)gFq(a.s,d[c]||null);b.errors.length>0&&fFq(a.s,b.errors_base_dir,b.errors,false,0,true,b.type);b.running?Z9e(a.a.Wj,true,false):WOi(a.s.g);$Vg(a.s.g.c.g)}\nfunction NHq(a){try{if(a.c<0)return;if(!a.a.a){a.a.a=RJq(a.a.b.a);tqq(a.a,a.a.a)}switch(a.c){case 1:KFq(a.a.a,a.b[0]);break;case 2:a.a.a.K.Q7b();break;case 3:a.a.a.K.R7b();break;case 4:WFq(a.a.a,f3A,'');break;case 5:WFq(a.a.a,'build-binary-package','');break;case 6:WFq(a.a.a,'build-full','');break;case 7:WFq(a.a.a,'build-incremental','');break;case 8:WFq(a.a.a,'build-source-package','');break;case 9:WFq(a.a.a,'check-package','');break;case 10:WFq(a.a.a,'clean-all','');break;case 11:RFq(a.a.a);break;case 12:SFq(a.a.a);break;case 13:TFq(a.a.a);break;case 14:HGm(a.a.a.k,new cGq((IFq(),HFq).Q8b()));break;case 15:WFq(a.a.a,'test-package','');break;case 18:a.a.a.K.T7b();}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].nn()}}\nvar pVB='roxygenize-package',qVB='Error Terminating Build',rVB='Unable to terminate build. Please try again.',sVB='Building package documentation',tVB='Terminating Build...';VNd(2335,1,cKy,nYi);_.rw=function oYi(a){WWi(this.a,a)};var Wyb=MAe(2335);VNd(8204,394,{},cGq);_.an=function dGq(a){(nQe(a),a)||BBi(3,(IFq(),HFq).D8b(),HFq.E8b()).hV()};var wic=MAe(8204);VNd(8205,1,hKy,eGq);_.nn=function fGq(){$Vg(this.a.s.g.c.g)};var xic=MAe(8205);VNd(8206,54,{},gGq);_.CA=function hGq(a){this.a.c=a;KGq(this.b,this.a.c)};var yic=MAe(8206);VNd(8197,1,hKy,GGq);_.nn=function HGq(){NFq(this.a)};var Mic=MAe(8197);VNd(8198,1,hKy,IGq);_.nn=function JGq(){IFq()};var Nic=MAe(8198);VNd(8196,1,cKy,LGq);_.rw=function MGq(a){KGq(this,a)};var Oic=MAe(8196);VNd(8199,1,hKy,NGq);_.nn=function OGq(){WFq(this.a,pVB,'')};var Pic=MAe(8199);VNd(255,1,wiz);_.ln=function QHq(){NHq((new RHq(this)).a)};VNd(7472,1,hKy,RHq);_.nn=function SHq(){NHq(this.a)};var kjc=MAe(7472);VNd(5605,1,h3A);_.D8b=function cIq(){return qVB};_.E8b=function dIq(){return rVB};_.H8b=function gIq(){return sVB};_.Q8b=function pIq(){return tVB};VNd(5604,1,h3A);_.D8b=function BIq(){return qVB};_.E8b=function CIq(){return rVB};_.H8b=function FIq(){return sVB};_.Q8b=function OIq(){return tVB};VNd(5603,1,h3A);_.D8b=function $Iq(){return 'Erreur de fin de construction'};_.E8b=function _Iq(){return 'Impossible de terminer la construction. Veuillez r\\xE9essayer.'};_.H8b=function cJq(){return 'Documentation du paquet de construction'};_.Q8b=function lJq(){return 'Terminer la construction...'};auy(hm)(15);\n//# sourceURL=rstudio-15.js\n")
