/*-- scss:defaults --*/

/* TODO: Customize presentation appearance by setting SCSS variables */
/* See https://quarto.org/docs/presentations/revealjs/themes.html#customizing-themes */

/*-- scss:rules --*/

/* TODO: Provide custom CSS rules */
