$wnd.rstudio.runAsyncCallback16("function VUt(a){this.a=a}\nfunction lSt(a,b){this.a=a;this.b=b;Wmf.call(this)}\nfunction PRt(a,b){a.a=b;hUt(a.q,LRt(a,null),a.a.file_path)}\nfunction RRt(a,b){b.slide_index==0&&qxq(a.q);a.a=b;hUt(a.q,LRt(a,null),a.a.file_path)}\nfunction YRt(a){yrh(JRt.Sqc(),Uoe());o5h(a.c,new j7h,false)}\nfunction MRt(a){var b;b=PCi(new QCi(0,JRt.Oqc()));DAm(a.o,new lSt(a,b))}\nfunction DAm(a,b){GFm(a,yzz,'close_presentation_pane',new xL,b)}\nfunction XRt(a){var b;a.K.S7b();!!a.a&&a.a.active&&(b=Wjf(Oh(a.q.d).contentWindow),!(!wCe(UNy,b)&&!wCe('javascript:void(0)',b)))&&PRt(a,a.a)}\nfunction RUt(a){var b,c;try{if(a.c<0)return;if(!a.a.a){a.a.a=FVt(a.a.b.a);tqq(a.a,a.a.a)}switch(a.c){case 1:MRt((c=a.a.a,a.b[0],c));break;case 2:RRt(a.a.a,a.b[0]);break;case 3:a.a.a.K.Q7b();break;case 4:a.a.a.K.R7b();break;case 5:XRt(a.a.a);break;case 6:YRt((b=a.a.a,a.b[0],b));break;case 7:a.a.a.K.T7b();}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].nn()}}\nvar uVB='Closing Presentation...',vVB='Opening Presentation...';VNd(8219,33,{},lSt);_.CA=function nSt(a){var b;o5h((b=this,a,b).a.c,new j7h,false)};_.BA=function mSt(a){dCi(this.b,a.a?a.a.c:a.d)};var MRc=MAe(8219);VNd(9089,1,YqB);_.Oqc=function SSt(){return uVB};_.Sqc=function XSt(){return vVB};VNd(9087,1,YqB);_.Oqc=function iTt(){return uVB};_.Sqc=function nTt(){return vVB};VNd(9088,1,YqB);_.Oqc=function ATt(){return 'Fermeture de la pr\\xE9sentation...'};_.Sqc=function FTt(){return 'Ouvrir la pr\\xE9sentation...'};VNd(466,1,wiz);_.ln=function UUt(){RUt((new VUt(this)).a)};VNd(7488,1,hKy,VUt);_.nn=function WUt(){RUt(this.a)};var mSc=MAe(7488);auy(hm)(16);\n//# sourceURL=rstudio-16.js\n")
