$wnd.rstudio.runAsyncCallback17("function CXt(a){this.a=a}\nfunction $6t(a){this.a=a}\nfunction a7t(a,b){this.a=a;this.b=b}\nfunction fOv(a,b){this.a=a;this.b=b}\nfunction BOv(a,b){this.a=a;this.b=b}\nfunction j2w(a,b){i2w();this.b=a;this.a=b}\nfunction XMw(a,b){ivk(a.w,b)}\nfunction mAv(a,b){XMw(a.vb,b)}\nfunction QVt(a){IBi(a.o,a.c,null)}\nfunction RVt(a){IBi(a.o,a.g,null)}\nfunction wWt(a){NWt(a.p,a.g,ARA,null)}\nfunction BWt(a){NWt(a.p,a.g,'next',null)}\nfunction DWt(a){NWt(a.p,a.g,'prev',null)}\nfunction EWt(a){NWt(a.p,a.g,'refresh',null)}\nfunction SVt(a){IBi(a.o,a.g+'?print-pdf',null)}\nfunction o2t(a,b){W0t();var c;c=a;uAv(c,b.a)}\nfunction fAv(a,b){gn((_m(),$m),new BOv(a,b))}\nfunction n2t(a,b){W0t();gn((_m(),$m),new a7t(b,a))}\nfunction axj(a){return {items:a}}\nfunction FWh(a){$wnd.desktop.setPresentationUrl(a)}\nfunction ivk(a,b){a.d.navigateToPresentationEditorLocation(b)}\nfunction uAv(a,b){b6w(a.A,Gwz,false)?Dyv(a,new fOv(a,b)):gTv(a.ab,b)}\nfunction KVt(a,b){wCe(b.substr(0,4),Iyy)||(b=CBm(a.q,b));return b}\nfunction PVt(a){!!a.b&&o5h(a.n,new j2w(Bnf(a.b.source_file),Wwj(a.d,a.a,a.e)),false)}\nfunction Xwj(a,b){var c,d,e,f;b==-1&&(b=a.auto_slide_level);f=Vwj(a,b,true);for(c=0;c<f.length;c++){e=f[c];for(d=0;d<e.items.length;d++){if(wCe(e.items[d].type,r9y))return c}}return 0}\nfunction Wwj(a,b,c){var d,e,f,g,h;c==-1&&(c=b.auto_slide_level);h=Vwj(b,c,false);if(a<h.length){g=h[a];zl(g.items,Zwj(g.items[0].row))}e=[];for(d=0;d<h.length;d++){g=h[d];for(f=0;f<g.items.length;f++){xl(e,g.items[f])}}return Ywj(e,b.auto_slide_level)}\nfunction gTv(a,b){var c,d,e,f,g,h,i;e=fTv(a).items;d=[];for(g=0;g<e.length;g++){h=e[g];wCe(h.type,r9y)||(d[d.length]=h,undefined)}c=-1;i=b.items;for(f=0;f<i.length;f++){if(wCe(i[f].type,r9y)){c=f;break}}if(c>=0){h=d[c];!!h&&gYu(a.a,h8w(h.row,0),true)}}\nfunction Vwj(a,b,c){var d,e,f,g,h,i;i=[];g=a.items;for(d=0;d<g.length;d++){f=g[d];e=i.length==0||wCe(f.type,_vy)||wCe(f.type,'hr')||wCe(f.type,yvy)&&f.level<=b;if(e){h=axj([]);xl(h.items,f);i[i.length]=h}else (c||!wCe(f.type,r9y))&&xl(i[i.length-1].items,f)}return i}\nfunction TVt(a,b){var c,d;c=b.a;d=KVt(a,c.url);a.p=Xwj(b.a.editor_state,b.a.slide_level);a.p==0&&a.d!=0&&(a.p=a.d);qxq(a.k);if(!wCe(d,a.g)||!a.k.p){a.g=d;a.c=d;a.b=c.quarto_navigation;a.a=c.editor_state;a.e=c.slide_level;(zVh()||!!$wnd.remoteDesktop)&&FWh(a.g);AWt(a.k,a.g,a.b)}else{EWt(a.k)}}\nfunction AWt(a,b,c){var d;(zVh()||!!$wnd.remoteDesktop)&&(y6e(b)?(Oh(a.d).removeAttribute(S8y),undefined):(Oh(a.d).setAttribute(S8y,jSB),undefined));a.b=b;a.p=null;a.g=null;d=Qh(a.d);d>0&&yi(a,new Ykf(hO(d*0.6666666666666666)+((a.L?Ph(a.L):0)+(a.P?Ph(a.P):0))));eUg(a.d,a.b);c.is_website?hum(a.j):gum(a.j,c.source_file,c.output_file);Moh(a.q)}\nfunction yXt(a){try{if(a.c<0)return;if(!a.a.a){a.a.a=bYt(a.a.b.a);tqq(a.a,a.a.a)}switch(a.c){case 1:a.a.a.K.Q7b();break;case 2:a.a.a.K.R7b();break;case 3:PVt(a.a.a);break;case 4:wWt(a.a.a.k);break;case 5:BWt(a.a.a.k);break;case 6:QVt(a.a.a);break;case 7:RVt(a.a.a);break;case 8:DWt(a.a.a.k);break;case 9:SVt(a.a.a);break;case 10:TVt(a.a.a,a.b[0]);break;case 11:EWt(a.a.a.k);break;case 12:a.a.a.K.S7b();break;case 13:a.a.a.K.T7b();}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].nn()}}\nVNd(305,1,wiz);_.ln=function BXt(){yXt((new CXt(this)).a)};VNd(7491,1,hKy,CXt);_.nn=function DXt(){yXt(this.a)};var BRc=MAe(7491);VNd(3291,1,cKy,$6t);_.rw=function _6t(a){n2t(this.a,a)};var tVc=MAe(3291);VNd(3290,1,Dwy,a7t);_.nn=function b7t(){o2t(this.a,this.b)};var uVc=MAe(3290);VNd(5065,1,hKy,fOv);_.nn=function gOv(){fAv(this.a,this.b)};var K9c=MAe(5065);VNd(5064,1,Dwy,BOv);_.nn=function COv(){mAv(this.a,this.b)};var V9c=MAe(5064);VNd(3339,myy,{},j2w);_._n=function k2w(a){var b,c;b=(c=Kcj(a.n,this.b),!!c&&aO(c,76)?c:(Ecj(),ycj));$au(a.b,this.b,b,new $6t(this))};_.bo=function m2w(){return h2w};_.ao=function l2w(){return h2w};var crd=MAe(3339);auy(hm)(17);\n//# sourceURL=rstudio-17.js\n")
