$wnd.rstudio.runAsyncCallback7("function Dit(a){this.a=a}\nfunction Jht(a,b){this.a=a;this.b=b}\nfunction Fht(a,b){b==null?zaf(a,Dht.Jmc()):zaf(a,b)}\nfunction Wht(a,b){yi(a.d,new jlf(false));Nht(a.d,b,0)}\nfunction fEm(a,b){GFm(a,yzz,'markers_tab_closed',new xL,b)}\nfunction Ght(a,b){var c,d,e,f,g,h,i;d=a.s;Cie(d.p);for(g=b,h=0,i=g.length;h<i;++h){f=g[h];c=new Jht(a,f);e=new dQd(bQd(new cQd,f).a.a);enh(d,new Uje(e,c))}}\nfunction Nht(a,b,c){var d;FKj(a.b);Fht(a.c,null);Ght(a.c,nN(hN(c1,1),Luy,2,6,[]));if(b.markers){d=b.markers;JKj(a.b,null,d.base_path,d.markers,c,false);Ght(a.c,Iyf(b.names));Fht(a.c,d.name)}}\nfunction Vht(a,b){var c,d,e,f,g;g=b.a.markers_state;if(g.markers){yi(a.d,new jlf(true));Nht(a.d,b.a.markers_state,b.a.auto_select);e=g.markers;if(e){d=e.markers;if(d.length>0){f=null;c=b.a.auto_select;c==1?(f=d[0]):c==2&&(f=nKj(d));!!f&&Fcj(a.b,Bnf(f.path),s1e(f.line,f.column),false)}}}else{yi(a.d,new clf)}}\nfunction zit(a){try{if(a.c<0)return;if(!a.a.a){a.a.a=Mit(a.a.b.a);tqq(a.a,a.a.a)}switch(a.c){case 1:Uht(a.a.a);break;case 2:a.a.a.K.Q7b();break;case 3:a.a.a.K.R7b();break;case 4:fEm(a.a.a.c,new aRh);break;case 5:Vht(a.a.a,a.b[0]);break;case 6:a.a.a.K.S7b();break;case 7:a.a.a.K.T7b();break;case 8:Wht(a.a.a,a.b[0]);}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].nn()}}\nVNd(9104,1,Dwy,Jht);_.nn=function Kht(){yE(this.a,this.b)};var $Kc=MAe(9104);VNd(441,1,wiz);_.ln=function Cit(){zit((new Dit(this)).a)};VNd(7245,1,hKy,Dit);_.nn=function Eit(){zit(this.a)};var lLc=MAe(7245);auy(hm)(7);\n//# sourceURL=rstudio-7.js\n")
